/*
curve - performs curve approximation with solid or dashed lines

reference: mcconalogue, d. j. (1970). a quasi-intrinsic scheme for
passing a smooth curve through a discrete set of points, the computer
journal, vol 13 no 4, pp 392-6.
*/

#ident "$Id: curve.c,v 1.3 2007/05/22 15:33:04 ccrawford Exp $"

#include <math.h>
#include "crcplot.h"

void curve(x,y,ne,delta)
float *x,*y;	/* array of points to be joined by a smooth curve */
int ne;		/* # points in array; <0 scale factors in +1,+2 
		 elements of array. >0 points already scaled
		 for plotting */
double delta;	/* segment length for incremental approximation 
		 to curve; <0 dashed lines with 'delta' length.
		 >0 solid line */
{
	int mpen,k,ipen;
	float xoff,yoff,xfac,yfac,dltsq,x2=-1,y2=-1,dltx1,dlty1,dltx2,dlty2;
	float y1=-1,t1,t2,t3,x1=-1,e,f,g,c2,s2,u,v,a,c1=-1,s1=-1,h=-1,b,t,c;
/*
	check parameters
*/
	plots_open_check("curve");

	if(ne == 0 || delta == 0.0)return;
/*
	set scale factors
*/
	if(ne < 0){
		ne = -ne;
		xoff = x[ne];
		yoff = y[ne];
		xfac = x[ne+1];
		yfac = y[ne+1];
	}else{
		xoff = 0.0;
		yoff = 0.0;
		xfac = 1.0;
		yfac = 1.0;
	}
/*
	check for solid or dashed lines
*/
	if (delta <= 0.0){
		delta = -delta;
		mpen = 5;
	}else{
		mpen = 4;
	}
/*
	initialize
*/
	k = -1;
	ipen = 3;
	dltsq = delta * delta;
/*
	begin main loop. (x1,y1) is joined to (x2,y2) by arc with direction
	cosines (c1,s1) and (c2,s2) at end points. final values for previous
	arc are taken as initial values for new arc.
*/
	while(++k < ne){
/*
	compute values for new end point
*/
	x2 = (x[k] - xoff) / xfac;
	y2 = (y[k] - yoff) / yfac;

	if(k == 0){ 				/* first data point */
		dltx1 = (x[1] - x[0]) / xfac;
		dlty1 = (y[1] - y[0]) / yfac;
		dltx2 = (x[2] - x[1]) / xfac;
		dlty2 = (y[2] - y[1]) / yfac;
		t1 = dltx1*dltx1 + dlty1*dlty1;
		t2 = dltx2*dltx2 + dlty2*dlty2;
		t3 = 2.0 * sqrt (t1*t2);
		t1 = -t1;
		t2 = t3 + t2;
	}else if(k < ne-1){ 			/* intermediate data point */
		dltx1 = x2 - x1;
		dlty1 = y2 - y1;
		dltx2 = (x[k+1] - x[k]) / xfac;
		dlty2 = (y[k+1] - y[k]) / yfac;
		t1 = dltx1*dltx1 + dlty1*dlty1;
		t2 = dltx2*dltx2 + dlty2*dlty2;
	}else{ 					/* last data point */
		dltx1 = x1 - (x[k-2] - xoff) / xfac;
		dlty1 = y1 - (y[k-2] - yoff) / yfac;
		dltx2 = x2 - x1;
		dlty2 = y2 - y1;
		t1 = dltx1*dltx1 + dlty1*dlty1;
		t2 = dltx2*dltx2 + dlty2*dlty2;
		t3 = 2.0 * sqrt (t1*t2);
		t2 = -t2;
		t1 = t3 + t1;
	}
/*
	set up to get integration parmater Tk
*/
	e = dltx1 * t2 + dltx2 * t1;
	f = dlty1 * t2 + dlty2 * t1;
	g = sqrt (e*e + f*f);
	if (g != 0.0) g = 1.0 / g;
	c2 = g * e;
	s2 = g * f;

	if (k == 0){
		plot (x2, y2, ipen);
		h = delta;
		ipen = 2;
		goto setup;
	}

	u = x2 - x1;
	v = y2 - y1;
	g = u*u + v*v;
	a = c1 * c2 + s1 * s2;
/* 
	check whether (x2,y2) is more than 'delta' away from (x1,y1) 
*/
	if(g < dltsq){
			/* coincident points or discontinuity in tangent */
		if (g <= 0.0 || a > 0.99996){
					/* (x1,y1) & (x2,y2) are less than 
					'delta' apart, skip to next point */
			
			goto setup;
		}
		plot (x2, y2, ipen);
		h = delta;
		ipen = 2;
		goto setup;
	}
/*
	calculate coefficients of cubics for x & y
*/
	a = 7.0 - a;
	e = c1 + c2;
	f = s1 + s2;
	b = u * e + v * f;
	t = sqrt (b*b + 2.0*a*g);
	c = (t + b) / g;
	t = 3.0 * (t - b) / a;
	g = c / 12.0;
	a = g * (c*u - 3.0*e);
	b = g * (c*v - 3.0*f);
	u = g * (c2 - c1) + a;
	v = g * (s2 - s1) + b;
	c = -c / 9.0;
	a = a * c;
	b = b * c;
	g = h;
/*
x & y coordinates of arc are given as cubics in a parameter [0,t] for fixed
g. the increment is delta. g initially is set to space the first
point of the new arc at distance delta from the last point of the
previous arc.
*/

/*
	generate segments of arc approximation
*/
	do{
		e = g * (g * (g*a + u) + c1) + x1;
		f = g * (g * (g*b + v) + s1) + y1;
		plot (e, f, ipen);
		ipen = mpen - ipen;
		g = g + delta;
		h = g - t;
	}while (h <= 0.0);
/*
	arc from (x1,y1) to (x2,y2) complete. set up for next arc
*/
setup:
	if(k >= 0){
		x1 = x2;
		y1 = y2;
		c1 = c2;
		s1 = s2;
	}
/*
	end while over all points
*/
	}
/*
	close to last point of curve
*/
	plot(x2,y2,ipen);
}
