/*
disk - handle disk access

parse_fclose - fclose with error checking
parse_fopen - fopen with error checking
parse_popen - popen with error checking
parse_pclose - pclose with error checking
parse_fseek - fseek with error checking
parse_fread - fread with error checking
parse_fwrite - fwrite with error checking
parse_rblock - rdos read a block
parse_wblock - rdos write a block
parse_fread_block - fread at certain block
parse_urline - read line of text from a unit
parse_rline - read line of text from stdin

internal support routines (static for now):
parse_fn_add - add a filename to linked list
parse_fn_delete - delete a filename from linked list
parse_fn_name - find filename associated w/ file descriptor
parse_fn_error - print error message and filename and then exit
*/

#ident "$Id: disk.c,v 1.8 2007/05/22 15:51:51 ccrawford Exp $"

#include <string.h>
#include <stdlib.h>
#include "parse.h"

#ifndef NT
#include <unistd.h> 
#else
#define popen _popen
#define pclose _pclose
#endif

struct pfn{		/* linked list for fn* routines */
	FILE *fd;
	char *name;
	struct pfn *next;
};

static int pfn_set;		/* 1= fd's set in following structure */
static struct pfn pfn_stderr = {NULL,"stderr",NULL};
static struct pfn pfn_stdout = {NULL,"stdout",&pfn_stderr};
static struct pfn pfn_stdin = {NULL,"stdin",&pfn_stdout};
static struct pfn *fn_root = &pfn_stdin;

static void 
fill_pfn (void)
{
	if(!pfn_set){
		pfn_stderr.fd = stderr;
		pfn_stdout.fd = stdout;
		pfn_stdin.fd = stdin;
		pfn_set = 1;
	}
}

char *
parse_fn_name (FILE *fd)
{
	struct pfn *p = fn_root;

	fill_pfn();

	while(1){
		if(p->fd == fd) return(p->name);
		if(p->next == NULL) return("File_Not_Registered");
		p = p->next;
	};
}

static void 
parse_fn_delete (FILE *fd)
{
	struct pfn *p = fn_root,*q;

	fill_pfn();

	if(fd == stdin || fd == stdout || fd == stderr) return;

	q = NULL;

	while(1){
		if(p->fd == fd) break;
		if(p->next == NULL) return;
		q = p;
		p = p->next;
	};

	if(q == NULL) parse_er("parse_fn_delete: q == NULL");

	q->next = p->next;
	parse_free(p->name);
	parse_free((char *)p);
}

static void 
parse_fn_add (FILE *fd, char *name)
{
	struct pfn *p = fn_root, *q;

	fill_pfn();

	if(fd == stdin || fd == stdout || fd == stderr) return;
	parse_fn_delete(fd);

	while(p->next != NULL) p = p->next;

	q = (struct pfn *) parse_malloc(sizeof(struct pfn));
	p->next = q;
	q->fd = fd;
	q->next = NULL;
	q->name = (char *)parse_malloc(strlen(name)+1);
	strcpy(q->name,name);
}

void 
parse_fn_error(char *s,FILE *fd)
{
	fprintf(stderr,"%s file: %s\n",s,parse_fn_name(fd));
	exit(1);
}


FILE *
parse_fopen (char *s, char *mode)
{
	FILE *stream;
	char *sp;

#ifndef NT
	sp = parse_root(s);
#else
	sp = s;
#endif

	if(!strcmp(mode,"w") || !strcmp(mode,"wb")) unlink(sp);
	if((stream = fopen(sp,mode)) == NULL){
		fprintf(stderr,"Can't open: %s\n",sp);
		exit(1);
	}
	parse_fn_add(stream,s);
	return(stream);
}

#ifndef NOPIPE
FILE *
parse_popen (char *s, char *mode)
{
	FILE *stream;

	if((stream = popen(s,mode)) == NULL){
		fprintf(stderr,"Can't pipe open: %s\n",s);
		exit(1);
	}
	parse_fn_add(stream,s);
	return(stream);
}

void 
parse_pclose (FILE *s)
{
	if(pclose(s) == -1){
		parse_fn_error("parse_pclose: can't close",s);
		exit(1);
	}
	parse_fn_delete(s);
}
#endif

void 
parse_fclose (FILE *s)
{
	if(fclose(s) == EOF){
		parse_fn_error("parse_fclose: can't close",s);
		exit(1);
	}
	parse_fn_delete(s);
}

void 
parse_fseek (FILE *unit, long byte, char *type)
{
	int m;

	if(!strcmp(type,"b")){
		m = 0;
	}else if(!strcmp(type,"c")){
		m = 1;
	}else if(!strcmp(type,"e")){
		m = 2;
	}else{
		fprintf(stderr,"parse_fseek: illegal mode: %s file: %s",
			type,parse_fn_name(unit));
		exit(1);
	}

	if(-1 == fseek(unit,byte,m))
		parse_fn_error("parse_fseek: seek error",unit);
}

void 
parse_fread (void *ptr, int size, int n, FILE *unit)
{
	if(n != (int) fread(ptr,size,n,unit))
		parse_fn_error("parse_fread: read error",unit);
}

void 
parse_fwrite (void *ptr, int size, int n, FILE *unit)
{
	if(n < 0){
		parse_fn_error("parse_fwrite: negative count",unit);
	}else if(n>0 && n != (int) fwrite(ptr,size,n,unit)){
		parse_fn_error("parse_fwrite: write error",unit);
	}
}

void 
parse_fread_block (void *ptr, int size, int n, FILE *unit, int blk)
{
	parse_fseek(unit,(long) blk * 512,"b");
	parse_fread(ptr,size,n,unit);
}

void 
parse_rblock (FILE *unit, int start, void *data, int count)		
{
	parse_fseek(unit,(long) (start * 512L),"b");
	parse_fread(data,1,count*512,unit);
}

void 
parse_wblock (FILE *unit, int start, void *data, int count)
{
	parse_fseek(unit,(long) (start * 512L),"b");
	parse_fwrite(data,1,count*512,unit);
}

int 
parse_urline (FILE *u, char *s)
{
	char *ret;
	ret = fgets(s,PARSE_MAX_LINE,u);	
	while(*s){
		if(*s == '\n' || *s == '\r') *s = 0;
		s++;
	}
	return(ret == NULL);	/* 1 = eof */
}

int 
parse_rline (char *s)
{
	return(parse_urline(stdin,s));
}

/*
	debugging and testing routines 
*/
#ifdef PARSE_FN_TEST
int 
parse_fn_print (void)
{
	struct pfn *p = fn_root;

	fill_pfn();

	while(1){
		puts(p->name);
		if(p->next == NULL) break;
		p = p->next;
	};
	putchar('\n');
}

int 
main (void)
{
	FILE *fd,*fd2;;

	fd = parse_fopen("/dev/zero","r");
	fd2 = parse_fopen("/dev/zero","r");

	parse_fn_print();
	parse_fn_add(fd,"t.c");
	parse_fn_add(fd2,"t2.c");
	parse_fn_print();
	parse_fn_delete(fd);
	parse_fn_print();
	puts(parse_fn_name(fd2));
	puts(parse_fn_name(stdout));
	puts(parse_fn_name(fd));
}
#endif
