/*
	env - handle environment command
*/

#ident "$Id: env.c,v 1.14 2011/09/14 13:37:12 carl Exp $"

#include "xpic.h"

int current_env;			// current env

static
struct {
	char pathname[MAXPATHLEN];	// working directory
	int itype;			// image type
	int isub;			// image sub-type
	int file_open;			// file open
	char ifn[KB_BUF];		// input filename
	int current_image;		// active image number
	int images_total;		// total images in file
	FILE *image_fd;			// file descriptor for file
#ifdef GZMI
	gzFile *gz_image_fd;		// file descriptor for file
#endif
	int cen_set,cen_x,cen_y;	// for center command
	int cor_set,cor_x,cor_y;	// for corner command
	float pixel_dx;			// pixel size in x
	float pixel_dydx;		// ratio pixel size y/x
	int current_nx,current_ny;	// for scroll command
} envs[ENV_N];

static void
env_syntax(void)
{
	fputs("Synopsis: manage environments\n",ttyout);
	min_abbreviation();
	fputs("Related: cd, pwd\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("environment          * print environment contents\n",ttyout);
	fputs("                     * current environment indicated with *\n",ttyout);
	fputs("environment [-d] #   * switch to environment # (1-10)\n",ttyout);
	fputs("                     * -d = duplicate present environment\n",ttyout);
}

void
env_push(int e)
{
	if(NULL == getcwd(envs[e].pathname,MAXPATHLEN)) perror("xpic: env_push: getcwd error");
	strncpy(envs[e].ifn,ifn,KB_BUF);
	envs[e].itype = itype;
	envs[e].isub = isub;
	envs[e].file_open = file_open;
	envs[e].current_image = current_image;
	envs[e].images_total = images_total;
	envs[e].image_fd = image_fd;
#ifdef GZMI
	envs[e].gz_image_fd = gz_image_fd;
#endif
	envs[e].cen_set = cen_set;
	envs[e].cen_x = cen_x;
	envs[e].cen_y = cen_y;
	envs[e].cor_set = cor_set;
	envs[e].cor_x = cor_x;
	envs[e].cor_y = cor_y;
	envs[e].pixel_dx = pixel_dx;
	envs[e].pixel_dydx = pixel_dydx;
	envs[e].current_nx = current_nx;
	envs[e].current_ny = current_ny;

	if(itype == I_MI){
		mi_push(e);
	}
#ifdef GZMI
	else if(itype == I_GZMI){
	  gzmi_push(e);
	}
#endif
}

void
env_pop(int e, short verbose)
{
	/*parameter verbose, if true, prints status information for environment switching*/
	if(chdir(envs[e].pathname) == -1){
		fprintf(ttyout,"%s: %s\n",envs[e].pathname,strerror(errno));
	}

	strncpy(ifn,envs[e].ifn,KB_BUF);

	itype = envs[e].itype;
	isub = envs[e].isub;
	file_open = envs[e].file_open;
	current_image = envs[e].current_image;
	images_total = envs[e].images_total;
	image_fd = envs[e].image_fd;
#ifdef GZMI
	gz_image_fd = envs[e].gz_image_fd;
#endif
	cen_set = envs[e].cen_set;
	cen_x = envs[e].cen_x;
	cen_y = envs[e].cen_y;
	cor_set = envs[e].cor_set;
	cor_x = envs[e].cor_x;
	cor_y = envs[e].cor_y;
	pixel_dx = envs[e].pixel_dx;
	pixel_dydx = envs[e].pixel_dydx;
	current_nx = envs[e].current_nx;
	current_ny = envs[e].current_ny;


	if(itype == I_MI){
		mi_pop(e);
	}
#ifdef GZMI
	else if(itype == I_GZMI) {
	  gzmi_pop(e);
	}
#endif
	if(verbose){
		fprintf(ttyout,"env: %d: %s/",e + 1,envs[e].pathname);
		if(envs[e].file_open){
			fputs(envs[e].ifn,ttyout);
		}else{
			fputs("<no file>",ttyout);
		}
		fputs("\n",ttyout);
		fflush(ttyout);
	}
}

static void
env_new()
{
	int new;
	int dup;

	if(argc == 3){
		if(strcmp(argv[1],"-d")){
			env_syntax();
			return;
		}else{
			dup = 1;
		}
	}else{
		dup = 0;
	}

	/*This part doesn't let you use the -d option with a + or -
	  We *know about this* but it's been left for future work.*/
	  
	if(!strcmp(argv[1],"+")){
		env_inc(1);
		return ;
	}
	if(!strcmp(argv[1],"-")){
		env_dec(1);
		return ;
	}
	new = parse_atof(argv[1+dup]);
	if(parse_atof_error()) return;

	if(new < 1 || new > ENV_N){
		fputs("bad envrionment\n",ttyout);
		return;
	}
	new--;

	env_push(current_env);
	if(dup == 1){
		if(envs[new].file_open){
			env_pop(new,1);
			close_file();
			env_pop(current_env,1);
		}
		file_open = 0;
	}else{
		env_pop(new,1);
	}
	current_env = new;
	cwd2kb = 1;
}

void
env_init(void)
{
	int i;

	for(i=0;i<ENV_N;i++){
		env_push(i);
		envs[i].file_open = 0;
	}
	current_env = 0;
}

static void
env_print(void)
{
	int i;

	env_push(current_env);

	for(i=0;i<ENV_N;i++){
		fprintf(ttyout,"%2d",i+1);
		if(i == current_env){
			fputs("* ",ttyout);
		}else{
			fputs("  ",ttyout);
		}
		fprintf(ttyout,"%s ",envs[i].pathname);
		if(envs[i].file_open){
			fputs(envs[i].ifn,ttyout);
		}else{
			fputs("<no file>",ttyout);
		}

		if(envs[i].itype == I_MI){
			fputs(" MI",ttyout);
		}else if(envs[i].itype == I_LM){
			fputs(" LM",ttyout);
			switch(envs[i].isub){
				case I_LM_HIGH: fputs("-high",ttyout); break;
				case I_LM_LOW: fputs("-low",ttyout); break;
				case I_LM_PROJ: fputs("-proj",ttyout); break;
			}
		}else{
			fputs("?",ttyout);
		}
		fputc('\n',ttyout);
	}
}

void
cmd_env(void)
{
	if(argc < 1 || argc > 3){
		env_syntax();
		return;
	}

	if(argc == 1){
		env_print();
	}else{
		env_new();
		write_env_panel(current_env);
	}
}

void
env_inc(short verbose)
{
	if(current_env + 1 < ENV_N){
		env_push(current_env);
		env_pop(++current_env,verbose);
		cwd2kb = 1;
	}else{
		env_push(current_env);
		env_pop(current_env=0,verbose);
		cwd2kb = 1;
	}
}

void
env_dec(short verbose)
{
	if(current_env + 1 > 1){
		env_push(current_env);
		env_pop(--current_env,verbose);
		cwd2kb = 1;
	}else{
		env_push(current_env);
		env_pop(current_env=ENV_N-1,verbose);
		cwd2kb = 1;
	}
}

void
env_upd_dir(int envdest, char *src)
{
	/*This function updates the directory of the specified environment.
	  It does not change the directory.  The idea is to call this first,
	  then switch to the environment and let env_pop change the directory.
	  
	  This is a bit of a kludge.  It was necessary for expand_open() (images.c)
	  to work to spec, and in that case it is unnecessary to change the directory
	  here.  Don't misuse this ;) It could make things messy*/
	  
	if(0 <= envdest && ENV_N > envdest){
		strcpy(envs[envdest].pathname,src);
	}else{
		fprintf(ttyout,"env_cp_dir: environment out of range: coudnt copy to %d (max %d)\n",envdest,ENV_N);
		fflush(ttyout);
	}
}
