/*
misc - miscellaneous commands
*/

#ident "$Id: misc.c,v 1.20 2007/05/22 13:22:45 ccrawford Exp $"

#include "xpic.h"
#include <pwd.h>

static void
expression_syntax(void)
{
	fputs("Synopsis: evaluate expressions\n",ttyout);
	min_abbreviation();
	fputs("Syntax:\n",ttyout);
	fputs("expression exp       * evaluate exp; exp can contain +-*/()\n",ttyout);
	fputs("                     * spaces not allowed\n",ttyout);
	fputs("                     * expressions allowed with other xpic commands\n",ttyout);
}

void
cmd_expression(void)
{
	float e;
	int f;

	if(argc != 2){
		expression_syntax();
		return;
	}
	e = parse_atof(argv[1]);
	f = e;
	if(!parse_atof_error()){
		if(f == e){
			fprintf(ttyout,"%d\n",f);
		}else{
			fprintf(ttyout,"%f\n",e);
		}
	}
}

void
quit_guts(void)
{
	display_destroy();
	display_clean_up();
}

static void
exit_syntax(void)
{
	fputs("Synopsis: exit xpic\n",ttyout);
	min_abbreviation();
	fputs("Related: pause, source\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("exit      * exit xpic\n",ttyout);
}

void
cmd_exit(void)
{
	if(check_source("exit")) return;
	if(argc != 1){
		exit_syntax();
		return;
	}
	quit_guts();
	exit(0);
}

static void
cd_syntax(void)
{
	fputs("Synopsis: change working directory\n",ttyout);
	min_abbreviation();
	fputs("Related: pwd, environment\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("cd path      * change to directory path\n",ttyout);
	fputs("             * can use ~/ and ~user/ in path\n",ttyout);
	fputs("cd           * change to HOME directory\n",ttyout);
}

void
cmd_cd(void)
{
	char *home;
	static char path[BUFL];
	struct passwd *pw;
	char *s,*sp;
	int uid;

	if(check_source("cd")) return;
	if(argc > 2 || argc < 1){
		cd_syntax();
		return;
	}

	if(argc == 1){
		if((home = getenv("HOME")) == NULL){
			fprintf(ttyout,"No HOME environment variable\n");
			return;
		}
	}else{
		home = argv[1];
		s = home;
		if(*home == '~'){
			if(isalpha(s[1])){
				sp = ++s;
				while(isalnum(*s)) s++;
				strncpy(path,sp,s-sp);
				path[s-sp] = 0;
				if(NULL == (pw = getpwnam(path))){
					fprintf(ttyout,"Don't know user: %s\n",path);
					return;
				}
			}else{
				uid = geteuid();
				if(NULL == (pw = getpwuid(uid))){
					fprintf(ttyout,"Don't know uid: %d\n",uid);
					return;
				}
				s++;
			}
			strcpy(path,pw->pw_dir);
			strncat(path,s,BUFL-strlen(path));
		}else{
			strncpy(path,s,BUFL);
		}
		home = path;
	}

	if(chdir(home) == -1){
		fprintf(ttyout,"%s: %s\n",argv[1],strerror(errno));
	}else{
		cwd2kb = 1;
	}
}

void
do_clear(int load)
{
	register int i,j;
	register CPIXEL *ip,*ipp;

	if(da_x == 0 && da_y == 0 && 
	  da_xw == canvas_size_x && da_yw == canvas_size_y){
		memset((char *)image16,0,sizeof(CPIXEL)*canvas_size_x*canvas_size_y);
		if(load) load_pix(0);
	}else{
		ip = image16 + da_y*canvas_size_x + da_x;
		for(j=0;j<da_yw;j++){
			for(ipp=ip,i=0;i<da_xw;i++){
				*ipp++ = 0;
			}
			ip += canvas_size_x;
		}
		if(load) load_pix(1);
	}
}

static void
clear_syntax(void)
{
	fputs("Synopsis: clear (erase) display\n",ttyout);
	min_abbreviation();
	fputs("Related: reset, set\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("clear          * clear active canvas\n",ttyout);
	fputs("clear -a       * also clear annotations and call reset\n",ttyout);
}

void
cmd_clear(void)
{
	int synerr=0;
	int clear_all=0;

	if(argc == 2){
		synerr = strcmp(argv[1],"-a");
		clear_all = 1;
	}else if(argc < 1 || argc > 2){
		synerr = 1;
	}

	if(synerr){
		clear_syntax();
		return;
	}

	if(clear_all){
		reset();
		annotate_clear();
	}
	do_clear(1);
}


static void
pwd_syntax(void)
{
	fputs("Synopsis: print working directory\n",ttyout);
	min_abbreviation();
	fputs("Related: cd, environment\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("pwd           * print working directory directory\n",ttyout);
}

void
cmd_pwd(void)
{
	char pathname[MAXPATHLEN];

	if(argc != 1){
		pwd_syntax();
		return;
	}

	fprintf(ttyout,"pwd: %s\n",getcwd(pathname,MAXPATHLEN));
	cwd2kb = 1;
}

static void
shell_syntax(void)
{
	fputs("Synopsis: executes commands or shells\n",ttyout);
	min_abbreviation();
	fputs("Related: source\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("!command args...     * executes command and returns\n",ttyout);
	fputs("!                    * brings up shell\n",ttyout);
	fputs("ls args...           * executes ls command\n",ttyout);
}

void
cmd_shell(void)
{
	if(argc != 1){
		shell_syntax();
		return;
	}
	cwd2kb = 1;
}

static void
status_syntax(void)
{
	fputs("Synopsis: print status information\n",ttyout);
	min_abbreviation();
	fputs("Related: set\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("status          * prints basic information\n",ttyout);
	fputs("status -a       * prints all (extended) information\n",ttyout);
}

void
cmd_status(void)
{
	char pathname[MAXPATHLEN];

	if(argc <= 0 || argc > 2 || (argc == 2 && strcmp(argv[1],"-a"))){
		status_syntax();
		return;
	}

	fprintf(ttyout,"Cwd: %s",getcwd(pathname,MAXPATHLEN));

	fputs(" Type: ",ttyout);
	if(itype == I_MI){
		fputs("MI",ttyout);
	}else if(itype == I_LM){
#ifdef MERLIN
		fputs("L3F",ttyout); 
#else
		fputs("LM",ttyout); 
#endif
		switch(isub){
			case I_LM_HIGH: fputs("-high",ttyout); break;
			case I_LM_LOW: fputs("-low",ttyout); break;
			case I_LM_PROJ: fputs("-proj",ttyout); break;
		}
#ifdef GZMI
	}else if(itype == I_GZMI){
	  fputs("GZMI",ttyout);
#endif
	}else{
		fputs("?",ttyout);
	}
	fputc('\n',ttyout);

	if(file_open){
		fprintf(ttyout,"File: %s Images: %d",ifn,images_total);
		if(current_image)
		fprintf(ttyout," Current: %d",current_image);
		fputc('\n',ttyout);
	}

	wl_status();
	fprintf(ttyout,", (x0,y0,xw,yw): (%d,%d,%d,%d)",
		da_x,da_y,da_xw,da_yw);

	if(cen_set) 
		fprintf(ttyout," Center: (%d,%d)",cen_x,cen_y);
	if(cor_set) 
		fprintf(ttyout," Corner: (%d,%d)",cor_x,cor_y);
	fputc('\n',ttyout);

	fprintf(ttyout,"Gamma: %.1f Zero: %d\n",gamma_display,zero_offset);

	if(argc != 1){
		display_status();
	}
}	

void
cmd_cine(void)
{
	if(check_source("cine")) return;
	cine_syntax();
	return;
}

static void
source_syntax(void)
{
	fputs("Synopsis: read xpic commands from file\n",ttyout);
	min_abbreviation();
	fputs("Related: pause, log, annotate \n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("source file        * read commands from file\n",ttyout);
}

void
cmd_source(void)
{
	if(check_source("source")) return;
	source_syntax();
	return;
}

static void
pause_syntax(void)
{
	fputs("Synopsis: pause execution of sourced files\n",ttyout);
	min_abbreviation();
	fputs("Related: source, log, annotate \n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("pause             * pause & wait for carriage return\n",ttyout);
	fputs("pause message...  * print message before pausing\n",ttyout);
}

void
cmd_pause(void)
{
	if(check_source("pause")) return;
	pause_syntax();
	return;
}

static void
log_syntax(void)
{
	fputs("Synopsis: logs (writes) keyboard commands to file\n",ttyout);
	min_abbreviation();
	fputs("Related: source, pause\n",ttyout);
	fputs("Syntax:\n",ttyout);

	fputs("log -w [!]file      * write commands to file\n",ttyout);
	fputs("                    * overwrite file with !\n",ttyout);
}

void
cmd_log(void)
{
	if(check_source("log")) return;
	log_syntax();
	return;
}

static void
help_syntax(void)
{
	fputs("Synopsis: print list of commands or help per command\n",ttyout);
	min_abbreviation();
	fputs("Syntax:\n",ttyout);
	fputs("help            * print list of xpic commands\n",ttyout);
	fputs("help command    * print syntax for command\n",ttyout);
	fputs("                * '*' is start of comment\n",ttyout);
}

void
cmd_help(void)
{
	int i,c,m,n;

	if(argc < 1 || argc > 2){
		help_syntax();
		return;
	}

	if(argc == 2){
		if((i = lookup(argv[1],1)) >= 0){
			cmd_num = i;
			argc = -1;
			(*clist[i].func)();
		}
	}else{
		for(m=i=0;i<ncom;i++){
			c = strlen(clist[i].name);
			if(c > m) m = c;
		}
		c = display_tty_width() - 2;
		n = c / (m + 1);
		m = -m;
		for(i=0;i<ncom;i++){
			fprintf(ttyout,"%*s%c",m,clist[i].name,
			 ((i+1)%n)? ' ' : '\n');
		}
		if(i%n)fputs("\n",ttyout);
	}
}

void
reset(void)
{
	da_x = 0;
	da_y = 0;
	da_xw = canvas_size_x;
	da_yw = canvas_size_y;
	cen_set = 0;
	cor_set = 0;
	note = 0;
}

static void
reset_syntax(void)
{
	fputs("Synopsis: turns off corner, center & subregion\n",ttyout);
	min_abbreviation();
	fputs("Related: corner, center, status\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("reset          * performs reset\n",ttyout);
}

void
cmd_reset(void)
{
	if(argc <= 0){
		reset_syntax();
		return;
	}

	reset();
}

static void
zero_syntax(void)
{
	fputs("Synopsis: manages zero (offset) for pixel reporting\n",ttyout);
	min_abbreviation();
	fputs("Related: window, level, wl\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("zero val        * sets offset to val\n",ttyout);
	fputs("zero            * prints offset\n",ttyout);
}

void
cmd_zero(void)
{
	float g;

	if(argc < 1){
		zero_syntax();
		return;
	}
	if(argc == 1){
		fprintf(ttyout,"%d\n",zero_offset);
	}else{
		g = parse_atof(argv[1]);
		if(parse_atof_error()) return;
		zero_offset = g;
	}
}

static void
header_syntax(void)
{
	fputs("Synopsis: print image headers\n",ttyout);
	min_abbreviation();
	fputs("Syntax:\n",ttyout);
	fputs("header           * print headers for all images\n",ttyout);
	fputs("header m n       * print headers for images m-n\n",ttyout);
}

void
cmd_header(void)
{
	int first,last;
	int i;
	int nx,ny;
	char label[PARSE_MI_LEN_LABEL+1];

	if(argc != 3 && argc != 1){ 
		header_syntax();
		return;
	}

	if(!file_open){
		fputs("Open a file first\n",ttyout);
		return;
	}

	if(argc == 3){
		first = parse_atof(argv[1]);
		if(parse_atof_error()) return;

		last = parse_atof(argv[2]);
		if(parse_atof_error()) return;
	}else{
		first = 1;
		last = images_total;
	}

	if(first < 1 || first > images_total){
		fprintf(ttyout,"Bad first image number\n");
		return;
	}

	if(last < 1 || last > images_total){
		fprintf(ttyout,"Bad last image number\n");
		return;
	}
				
	for(i=first;i<=last;i++){
		if(itype == I_LM){
			if(-1 == lm_image(&nx,&ny,i,&label[0])) return;
		}else if(itype == I_MI){
			if(-1 == mi_image(&nx,&ny,i,&label[0])) return;
#ifdef GZMI
		}else if(itype == I_GZMI){
			if(-1 == gzmi_image(&nx,&ny,i,&label[0])) return;
#endif
		}else{
			nx = ny = -1;
			*label = 0;
		}
		fprintf(ttyout,"Image: %d (%d,%d)",i,nx,ny);
		if(strlen(label)) fprintf(ttyout," \"%s\"",label);
		fputc('\n',ttyout);
	}
}

static void
panel_syntax(void)
{
	fputs("Synopsis: provide help for panel\n",ttyout);
	min_abbreviation();
	//fputs("Related: \n",ttyout);
	fputs("Sliders:\n",ttyout);
	fputs("  W:     * window width (see window, wl)\n",ttyout);
	fputs("  L:     * window level (see level, wl)\n",ttyout);
	fputs("Buttons:\n",ttyout);
	fputs("  +:     * next image (see +)\n",ttyout);
	fputs("  -:     * previous image (see -)\n",ttyout);
	fputs("  u:     * scroll up (see scroll)\n",ttyout);
	fputs("  d:     * scroll down (see scroll)\n",ttyout);
	fputs("  l:     * scroll left (see scroll)\n",ttyout);
	fputs("  r:     * scroll right (see scroll)\n",ttyout);
	fputs("  m:     * measure distance and angle (see measure)\n",ttyout);
	fputs("  F:     * fine window/level control (see window, level, wl)\n",ttyout);
	fputs("  C:     * coarse window/level control (see window, level, wl)\n",ttyout);
	fputs("  Browse * activate graphical file browser\n",ttyout);
	fputs("  e-     * decrement environment\n",ttyout);
	fputs("  e+     * increment environment\n",ttyout);
	fputs("  Quit   * quit xpic\n",ttyout);
}

void
cmd_panel(void)
{
	panel_syntax();
}

static void
mouse_syntax(void)
{
	fputs("Synopsis: provide help for mouse\n",ttyout);
	min_abbreviation();
	fputs("Related: roi, annotate, cursor, box\n",ttyout);
	fputs("Crosshair active:\n",ttyout);
	fputs("  Left click:     * report pixel location and value\n",ttyout);
	fputs("                  * or, deposit annotation (see annotate)\n",ttyout);
	fputs("  Middle click:   * no action\n",ttyout);
	fputs("  Right click:    * deposit cursor\n",ttyout);
	fputs("  No click:       * move cursor\n",ttyout);
	fputs("Box active:\n",ttyout);
	fputs("  Left click:     * report ROI information (see roi)\n",ttyout);
	fputs("                  * or, deposit annotation (see annotate)\n",ttyout);
	fputs("  Middle drag:    * change shape of box\n",ttyout);
	fputs("  Right click:    * deposit cursor\n",ttyout);
	fputs("  No click:       * move cursor\n",ttyout);
}

void
cmd_mouse(void)
{
	mouse_syntax();
}

static void
ls_syntax(void)
{
	fputs("Synopsis: list command\n",ttyout);
	min_abbreviation();
	fputs("Related: cd\n",ttyout);
	fputs("  ls:     * lists files and directories\n",ttyout);
	fputs("          *type !man ls for arguments\n",ttyout);
}

void
cmd_ls(void)
{
	if(argc < 1)
		ls_syntax();
}
