/*
set - handle set command
*/

#ident "$Id: set.c,v 1.17 2007/05/22 13:22:45 ccrawford Exp $"

#include "xpic.h"

#define O_BOOL 0		/* boolean type */
#define O_INT 1			/* integer type */
#define O_STR 2 		/* string type */
#define O_FLOAT 3 		/* float type */

#define O_RDNLY  16		/* read-only */

#define O_TMASK  0xF		/* mask for types */

static struct OTBL {
	char *fullname;
	char *abbrname;
	int type;
	void *addr;
	char *help;
} otbl[] = {
	{"autowl",	"awl",		O_BOOL,		&autowl,	"automatic window/level on image load [wl -a]"},
	{"autoclear",	"ac",		O_BOOL,		&autoclear,	"automatic display clearing on image load [clear]"},
	{"automagnify",	"am",		O_BOOL,		&automagnify,	"automatic magnify on image load [magnify]"},
	{"autoreset",	"ar",		O_BOOL,		&autoreset,	"automatic reset on file open [reset, open]"},
	{"boxred",	"br",		O_BOOL,		&boxred,	"red box cursors versus XORing"},
	{"diffoffset",	"do",		O_INT,		&diff_offset,	"offset for difference command [difference]"},
	{"dx",		"dx",		O_FLOAT,	&pixel_dx,	"x-pixel size for measurements [measure]"},
	{"dydx",	"dydx",		O_FLOAT,	&pixel_dydx,	"x/y-pixel ratio for measurements [measure]"},
	{"gamma",	"gamma",	O_FLOAT|O_RDNLY,&gamma_display,	"gamma setting for display [gamma]"},
	{"outline",	"ol",		O_INT,		&outline_start,	"first digital value for outline [wl -o]"},
	{"replicate",	"rep",		O_BOOL,		&mag_replicate,	"pixel replication for magnify [magnify]"},
	{"roiellipse",	"re",		O_BOOL,		&roiellipse,	"box/ellipse for roi [roi]"},
	{"roiminimum",	"rmin",		O_INT,		&roimin,	"pixel exclusion for roi [roi]"},
	{"scrollx",	"scx",		O_INT,		&scroll_x,	"horizontal (x) scroll increment [scroll]"},
	{"scrolly",	"scy",		O_INT,		&scroll_y,	"vertical (y) scroll increment [scroll]"},
	{"skip",	"sk",		O_INT,		&pm_skip,	"skip in +/- commands"},
	{"wltrack",	"wl",		O_BOOL,		&wltrack,	"track window/level slides [wl, panel]"},
	{"zero",	"zero",		O_INT,		&zero_offset,	"digital value for zero on display [zero]"}
};

static int nopt = sizeof(otbl) / sizeof(struct OTBL);

static void
set_set(void)
{
	int i;
	int no = 0;
	int num;
	float fnum;

	if(strlen(argv[1]) >= 2 && 
		argv[1][0] == 'n' && argv[1][1] == 'o') no = 2;

	if(!argv[1][no]){
		parse_uwtext(ttyout,"no option to set");
		return;
	}

	for(i=0;i<nopt;i++){
		if(!strcmp(argv[1]+no,otbl[i].fullname)) break;
		if(!strcmp(argv[1]+no,otbl[i].abbrname)) break;
	}
	if(i == nopt){
		parse_uwtext(ttyout,"illegal option");
		return;
	}

	if(otbl[i].type & O_RDNLY){
		parse_uwtext(ttyout,"option read-only");
		return;
	}

	switch(otbl[i].type & O_TMASK){
	case O_BOOL:
		if(argc != 2){
			parse_uwtext(ttyout,"boolean option syntax error");
			return;
		}
		*(int *)otbl[i].addr = no? 0 : 1;
		break;
	case O_INT:
		if(argc != 3){
			parse_uwtext(ttyout,"integer option syntax error");
			return;
		}
		num = parse_atof(argv[2]);
		if(parse_atof_error()) break;
		*(int *)otbl[i].addr = num;
		break;
	case O_FLOAT:
		if(argc != 3){
			parse_uwtext(ttyout,"float option syntax error");
			return;
		}
		fnum = parse_atof(argv[2]);
		if(parse_atof_error()) break;
		*(float *)otbl[i].addr = fnum;
		break;
	case O_STR:
		if(argc != 3){
			parse_uwtext(ttyout,"string option syntax error");
			return;
		}
		strncpy(otbl[i].addr,argv[2],OSL);
		break;
	default:
		parse_er("illegal type in set");
	}
}

static void 
set_print(void)
{
	int i;
	char s[140];
	char q[2*OSL];
	int m,n,c;
	char **qt;

	qt = (char **) parse_malloc(sizeof(char *)*nopt);

	for(m=i=0;i<nopt;i++){
		*q = 0;

		if(otbl[i].type & O_RDNLY) strcat(q,"*");

		switch(otbl[i].type & O_TMASK){

		case O_BOOL:
			if(!*(int *)otbl[i].addr) strcat(q,"no");
			strcat(q,otbl[i].fullname);
			break;

		case O_INT:
			strcat(q,otbl[i].fullname);
			strcat(q,"=");
			sprintf(s,"%d",*(int *)otbl[i].addr);
			strcat(q,s);
			break;

		case O_FLOAT:
			strcat(q,otbl[i].fullname);
			strcat(q,"=");
			sprintf(s,"%.2f",*(float *)otbl[i].addr);
			strcat(q,s);
			break;

		case O_STR:
			strcat(q,otbl[i].fullname);
			strcat(q,"=");
			strcat(q,otbl[i].addr);
			break;

		default:
			parse_er("illegal type in set_print");
		}

		c = strlen(q);
		if(c > m) m = c;
		qt[i] = parse_malloc_char(c+1);
		strcpy(qt[i],q);
	}

	c = display_tty_width();
	n = (c - 1) / (m + 1);
	m = -m;
	for(i=0;i<nopt;i++){
		fprintf(ttyout,"%*s%c",m,qt[i],
		 ((i+1)%n)? ' ' : '\n');
		 parse_free(qt[i]);
	}
	if(i%n)fputs("\n",ttyout);

	parse_free((void *) qt);
}

static void
set_syntax(void)
{
	int i;

	fputs("Synopsis: set/display variables\n",ttyout);
	min_abbreviation();
	fputs("Related: wl, window, level, difference, measure, scroll\n",ttyout);
	fputs("Syntax:\n",ttyout);
	fputs("set            * list variables\n",ttyout);
	fputs("set var value  * set variable var to value\n",ttyout);

	fputs("Variables:  fullname/abbreviation/type[*]: help\n",ttyout);
	fputs("            *=read-only; '[...]'=related commands\n",ttyout);

	for(i=0;i<nopt;i++){
		fputs(otbl[i].fullname,ttyout);
		fputc('/',ttyout);
		fputs(otbl[i].abbrname,ttyout);
		fputc('/',ttyout);

		switch(otbl[i].type & O_TMASK){
			case O_BOOL:
				fputs("boolean",ttyout);
				break;
			case O_FLOAT:
				fputs("float",ttyout);
				break;
			case O_INT:
				fputs("int",ttyout);
				break;
		}
		if(otbl[i].type & O_RDNLY) fputc('*',ttyout);
		fputs(": ",ttyout);
		fputs(otbl[i].help,ttyout);
		fputc('\n',ttyout);
	}
}

void
cmd_set(void)
{
	if(argc < 1){
		set_syntax();
	}else if(argc == 1){
		set_print();
	}else{
		set_set();
	}
}

#ifdef TEST_SET		/* convert to standalone for test purposes */

#define MAX_WORDS 20
main()
{
	char s[PARSE_MAX_LINE];
	strcpy(s,"set");
	s[0] = 0;

	while(1){
		parse_acs("command",s,NULL,s);
		parse_words(s,&argc,argv,MAX_WORDS);
		if(argc == 0 || strcmp("set",argv[0])){
			parse_wtext("command not set");
			continue;
		}
		set();
	}
}
#endif
